#include <stdio.h>
#include <stdint.h>
#include <time.h>
#include <sys/time.h>  
#include <math.h>
#include <stdbool.h>
#include <inttypes.h>
#include <stdlib.h> // For exit function
#include <unistd.h>    /* for getopt */
#include <string.h>    /* for strcmp */


#define MAX_PAYLOAD_BYTES 32
#define CRC_BYTES 4
//#define SYNCWORD 0x70df5040
//#define SYNCWORD 0x0405fd07
#define SYNCWORD 0xe0bfa020 // H1 syncword 0x70df5040 wordwise bitswapped
#define NRF_SYNCWORD 0xcacacaca // cx10 bind syncword
#define TRUE 1
#define FALSE 0

static uint32_t syncword;
static uint8_t payload[MAX_PAYLOAD_BYTES];
static uint8_t crc[CRC_BYTES];
static uint16_t crc_rec = 0;
static uint64_t offset = 0;

static uint8_t state;
enum {
  NOISE = 0,
  PREAMBLE_RECEIVED,
  SYNCWORD_RECEIVED,
  TRAILER_RECEIVED,
  PAYLOAD_RECEIVED,
  CRC_RECEIVED,
  NRF_SYNCWORD_RECEIVED
};

uint8_t reverse(unsigned char b) {
   b = (b & 0xF0) >> 4 | (b & 0x0F) << 4;
   b = (b & 0xCC) >> 2 | (b & 0x33) << 2;
   b = (b & 0xAA) >> 1 | (b & 0x55) << 1;
   return b;
}

#define                 P_KERMIT    0x8408
static int              crc_tabkermit_init      = FALSE;
static unsigned short   crc_tabkermit[256];

    /*******************************************************************\
    *                                                                   *
    *   static void init_crckermit_tab( void );                         *
    *                                                                   *
    *   The function init_crckermit_tab() is used to fill the array     *
    *   for calculation of the CRC Kermit with values.                  *
    *                                                                   *
    \*******************************************************************/

static void init_crckermit_tab( void ) {

    int i, j;
    unsigned short crc, c;

    for (i=0; i<256; i++) {

        crc = 0;
        c   = (unsigned short) i;

        for (j=0; j<8; j++) {

            if ( (crc ^ c) & 0x0001 ) crc = ( crc >> 1 ) ^ P_KERMIT;
            else                      crc =   crc >> 1;

            c = c >> 1;
        }

        crc_tabkermit[i] = crc;
    }

    crc_tabkermit_init = TRUE;

}  /* init_crckermit_tab */


    /*******************************************************************\
    *                                                                   *
    *   unsigned short update_crc_kermit( unsigned short crc, char c ); *
    *                                                                   *
    *   The function update_crc_kermit calculates a  new  CRC value     *
    *   based  on  the  previous value of the CRC and the next byte     *
    *   of the data to be checked.                                      *
    *                                                                   *
    \*******************************************************************/

unsigned short update_crc_kermit( unsigned short crc, char c ) {

    unsigned short tmp, short_c;

    short_c = 0x00ff & (unsigned short) c;

    if ( ! crc_tabkermit_init ) init_crckermit_tab();

    tmp =  crc       ^ short_c;
    crc = (crc >> 8) ^ crc_tabkermit[ tmp & 0xff ];

    return crc;

}  /* update_crc_kermit */




int main() {
  uint8_t bit, last_bit, byte, bytecount, preamble_bits, sync_pos, payload_size, obit = 0;
  uint16_t crc_kermit, low_byte, high_byte = 0;
  uint32_t bitcount, i = 0;
  uint64_t noisebits = 0;
  //bool preamble_received, sync = 0;

  uint32_t transitions[64];
  for (i=0; i<64; i++) transitions[i] = 0;

  // set initial state
  state = NOISE;
  noisebits = 0;

  while(!feof(stdin) ) {
    bit = fgetc(stdin); // the input file contains one bit per byte. This won't work for a real bitstream.

    // our byte to look at
    byte <<= 1; // shift left by one to make room for the new bit
    byte |= bit; // add new bit

    // calculate offset, so we know where to cut
    if (!(obit++%8)) { offset++; obit=0; }

    switch (state) {
      case NOISE:
        if (bit != last_bit) { // changing bits indicate a preamble of As or 5s.
          preamble_bits++;
        } else { // two consecutive 0s or 1s.
          transitions[preamble_bits]++; // stats
          if (preamble_bits >23) { // we have received a complete preamble of 3 bytes of As or 5s.
          //if (preamble_bits >7) { // we have received a complete preamble of 3 bytes of As or 5s.
            state = PREAMBLE_RECEIVED;

            printf("Noise: %08ld: ", noisebits);
            // the last bit is already part of the sync byte (the lsbit of the first byte)
            syncword = 0;
#if 1
            sync_pos = 31;
            printf("%x", last_bit);
            syncword |= last_bit<<sync_pos--; // store it
            printf("%x", bit);
            syncword |= bit<<sync_pos--; // store it
#else
            sync_pos = 31;
            syncword |= last_bit<<sync_pos--; // store it
            syncword |= bit<<sync_pos--; // store it
#endif
          }
          preamble_bits = 0; // reset preamble counter
          noisebits++;
        }
        break;

      case PREAMBLE_RECEIVED:
#if 1
        printf("%x", bit);
        syncword |= bit<<sync_pos; // store the current bit
        if (sync_pos == 0) {
#else
        if (sync_pos > 0) {
          printf("%x", bit);
          syncword |= bit<<sync_pos--; // store the current bit
        } else {
#endif
          printf(" %08x\n", syncword);
          if (syncword == SYNCWORD) {
            state = SYNCWORD_RECEIVED;
            bitcount = 0; // reset bitcount
          //} else if (syncword == NRF_SYNCWORD) {
          } else if (syncword == 0xe0bfa020) {
            state = NRF_SYNCWORD_RECEIVED;
            bitcount = 0; // reset bitcount
          } else {
            state = NOISE;
            //noisebits = 0;
          }
        }
        sync_pos--;
        break;

      case SYNCWORD_RECEIVED:
        // I assume that the trailer will be just like the preamble, since in the only set of test data I have, it is.
        if (bit != last_bit) {
          if (bitcount == 3) {
            state = TRAILER_RECEIVED;
          }
          bitcount++;
        } else {
          state = NOISE;
          //noisebits = 0;
        }
        break;


      case NRF_SYNCWORD_RECEIVED:
        if (bitcount < 32*8) { // size
          printf("%x", bit);
        } else {
          printf("\n");
          state = NOISE;
          //noisebits = 0;
          bytecount = 0;
          
        }
        bitcount++;
        break;

      case TRAILER_RECEIVED:
        if (bitcount < 12) { // size
          payload_size <<= 1;
          payload_size |= bit;
          if (bitcount == 11) {
            payload[bytecount++] = reverse(payload_size);
            printf("Payload Size: 0x%02x: ", reverse(payload_size));
          }
        } else {
          if ((bytecount <= (MAX_PAYLOAD_BYTES-1)) && (bytecount <= reverse(payload_size))) {
            if (!((bitcount + 5) % 8)) {
              payload[bytecount++] = reverse(byte);
              printf("%02x ", reverse(byte));
            } 
          } else {
            state = PAYLOAD_RECEIVED;
            bytecount = 0;
            //printf("\n");
            crc_kermit = 0;
            for (i=0; i <= payload_size; i++) {
              crc_kermit = update_crc_kermit(crc_kermit, payload[i]);
              //printf("%04x ", crc_kermit);
            }
            //printf("\n");
            low_byte   = (crc_kermit & 0xff00) >> 8;
            high_byte  = (crc_kermit & 0x00ff) << 8;
            crc_kermit = low_byte | high_byte;
            //printf("%04x ", crc_kermit);
            crc_tabkermit_init = FALSE;
            //printf("\n");
            printf("CRC: ");
          }
        }
        bitcount++;
        break;

      case PAYLOAD_RECEIVED:
        if (bytecount < 2) { 
          if (!((bitcount + 5) % 8)) {
            crc[bytecount++] = reverse(byte);
            printf("%02x ", reverse(byte));
            
          }
        } else {
          state = CRC_RECEIVED;
          bytecount = 0;

          crc_rec = (crc[0]<<8) | crc[1];

          printf("\ncrc_rec: %04x crc_kermit: %04x\n", crc_rec, crc_kermit);
          //printf("\n");
        }
        bitcount++;
        break;

      case CRC_RECEIVED:
        if (crc_rec == crc_kermit) printf("Valid data at offset 0x%08lx: ", offset);
        else printf("Corrupt data at offset 0x%08lx: ", offset);
        for (i=0; i<payload_size; i++) printf("%02x ", payload[i]);
        printf("\n\n");
        state = NOISE;
        noisebits = 0;
        break;
    }
    last_bit = bit;
	}
  printf("feddich\n");

  for (i = 32 ; i > 0 ; i--) {
    printf("%02d changes: %9d times\n", i, transitions[i]);
  }

/*
  for (i=0; i<MAX_PAYLOADS; i++)
    printf("Payload: ");
    for (j=0; j<MAX_PAYLOAD_BYTES; j++) {
      printf("%02x", payload[j]);
    }
    printf("\n");
*/

  return 0;
}

